from collections import OrderedDict

from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = OrderedDict(
        (
            ("{{first_name_female}} {{last_name_female}}", 0.97),
            ("{{prefix_female}} {{first_name_female}} {{last_name_female}}", 0.015),
            ("{{first_name_female}} {{last_name_female}} {{suffix}}", 0.02),
            (
                "{{prefix_female}} {{first_name_female}} {{last_name_female}} {{suffix}}",
                0.005,
            ),
        )
    )

    formats_male = OrderedDict(
        (
            ("{{first_name_male}} {{last_name_male}}", 0.97),
            ("{{prefix_male}} {{first_name_male}} {{last_name_male}}", 0.015),
            ("{{first_name_male}} {{last_name_male}} {{suffix}}", 0.02),
            (
                "{{prefix_male}} {{first_name_male}} {{last_name_male}} {{suffix}}",
                0.005,
            ),
        )
    )

    formats = formats_male.copy()
    formats.update(formats_female)

    # Names from
    # https://sk.wikipedia.org/wiki/Kategória:Krstné_mená

    first_names_male = (
        "Adam",
        "Adolf",
        "Adrián",
        "Alan",
        "Albert",
        "Albín",
        "Aleš",
        "Alex",
        "Alexander",
        "Alexej",
        "Alfonz",
        "Alfréd",
        "Alojz",
        "Ambróz",
        "Andrej",
        "Anton",
        "Ariel",
        "Arnold",
        "Áron",
        "Arpád",
        "Atila",
        "Augustín",
        "Aurel",
        "Bartolomej",
        "Belo",
        "Beňadik",
        "Benedikt",
        "Benjamín",
        "Bernard",
        "Blahoslav",
        "Blažej",
        "Bohdan",
        "Bohumil",
        "Bohumír",
        "Bohuslav",
        "Bohuš",
        "Boleslav",
        "Bonifác",
        "Boris",
        "Branislav",
        "Bruno",
        "Bystrík",
        "Ctibor",
        "Cyprián",
        "Cyril",
        "Dalibor",
        "Daniel",
        "Dávid",
        "Demeter",
        "Denis",
        "Dezider",
        "Dionýz",
        "Dobroslav",
        "Dominik",
        "Drahomír",
        "Drahoslav",
        "Dušan",
        "Eder",
        "Edmund",
        "Eduard",
        "Eliáš",
        "Emanuel",
        "Emil",
        "Erik",
        "Ernest",
        "Ervín",
        "Eugen",
        "Fedor",
        "Félix",
        "Ferdinand",
        "Filip",
        "Florián",
        "František",
        "Frederik",
        "Fridrich",
        "Gabriel",
        "Gašpar",
        "Gejza",
        "Gregor",
        "Gustáv",
        "Henrich",
        "Herbert",
        "Hubert",
        "Hugo",
        "Ignác",
        "Igor",
        "Iľja",
        "Imrich",
        "Ivan",
        "Izidor",
        "Jakub",
        "Ján",
        "Jarolím",
        "Jaromír",
        "Jaroslav",
        "Jerguš",
        "Jonáš",
        "Jozef",
        "Július",
        "Juraj",
        "Kamil",
        "Karol",
        "Kazimír",
        "Klement",
        "Kliment",
        "Koloman",
        "Konštantín",
        "Kornel",
        "Kristián",
        "Krištof",
        "Kvetoslav",
        "Ladislav",
        "Leonard",
        "Leopold",
        "Levoslav",
        "Libor",
        "Ľubomír",
        "Ľubor",
        "Ľuboslav",
        "Ľuboš",
        "Ludolf",
        "Ľudomil",
        "Ľudovít",
        "Lukáš",
        "Marcel",
        "Marek",
        "Marián",
        "Mário",
        "Maroš",
        "Martin",
        "Matej",
        "Matúš",
        "Maximilián",
        "Medard",
        "Melichar",
        "Metod",
        "Michal",
        "Mikuláš",
        "Milan",
        "Miloslav",
        "Miloš",
        "Miroslav",
        "Mojmír",
        "Móric",
        "Nathan",
        "Norbert",
        "Oldrich",
        "Oliver",
        "Ondrej",
        "Oskár",
        "Oto",
        "Pankrác",
        "Patrik",
        "Pavol",
        "Peter",
        "Pravoslav",
        "Prokop",
        "Radomír",
        "Radoslav",
        "Radovan",
        "Radúz",
        "Rastislav",
        "René",
        "Richard",
        "Róbert",
        "Roland",
        "Roman",
        "Rudolf",
        "Samuel",
        "Sebastián",
        "Sergej",
        "Servác",
        "Severín",
        "Silvester",
        "Slavomír",
        "Stanislav",
        "Svätopluk",
        "Svetozár",
        "Šimon",
        "Štefan",
        "Tadeáš",
        "Tarzícius",
        "Teodor",
        "Tibor",
        "Tichomír",
        "Timotej",
        "Tobiáš",
        "Tomáš",
        "Urban",
        "Václav",
        "Valentín",
        "Valér",
        "Vasil",
        "Vavrinec",
        "Vendelín",
        "Viktor",
        "Viliam",
        "Vincent",
        "Vít",
        "Víťazoslav",
        "Vladimír",
        "Vladislav",
        "Vlastimil",
        "Vojtech",
        "Vratislav",
        "Vratko",
        "Zdenko",
        "Zlatko",
        "Zoltán",
        "Zoran",
        "Žigmund",
    )

    # Names from
    # https://sk.wikipedia.org/wiki/Kategória:Ženské_mená

    first_names_female = (
        "Adela",
        "Adriána",
        "Agáta",
        "Agnesa",
        "Aiko",
        "Alana",
        "Albína",
        "Alena",
        "Alexandra",
        "Alexia",
        "Alica",
        "Alojzia",
        "Alžbeta",
        "Amália",
        "Anabela",
        "Andrea",
        "Anežka",
        "Angela",
        "Angelika",
        "Anna",
        "Antónia",
        "Aurélia",
        "Barbara",
        "Barbora",
        "Beáta",
        "Berta",
        "Bianka",
        "Bibiána",
        "Blanka",
        "Blažena",
        "Bohdana",
        "Bohumila",
        "Bohuslava",
        "Božena",
        "Božidara",
        "Branislava",
        "Brigita",
        "Bronislava",
        "Cecília",
        "Dagmara",
        "Dana",
        "Danica",
        "Daniela",
        "Darina",
        "Dáša",
        "Denisa",
        "Diana",
        "Dobromila",
        "Dobroslava",
        "Dominika",
        "Dorota",
        "Drahomíra",
        "Drahoslava",
        "Edita",
        "Ela",
        "Elena",
        "Eleonóra",
        "Elisa",
        "Eliška",
        "Elvíra",
        "Ema",
        "Emília",
        "Enna",
        "Erika",
        "Estera",
        "Etela",
        "Eugénia",
        "Eulália",
        "Eunika",
        "Eva",
        "Filoména",
        "Františka",
        "Frederika",
        "Gabriela",
        "Gertrúda",
        "Gizela",
        "Hana",
        "Hedviga",
        "Helena",
        "Henrieta",
        "Hermína",
        "Hilda",
        "Hortenzia",
        "Ida",
        "Ingrida",
        "Irena",
        "Iris",
        "Irma",
        "Ivana",
        "Iveta",
        "Ivica",
        "Ivona",
        "Izabela",
        "Jana",
        "Jarmila",
        "Jaromíra",
        "Jaroslava",
        "Jolana",
        "Jozefína",
        "Judita",
        "Júlia",
        "Juliana",
        "Justína",
        "Kamila",
        "Karina",
        "Karla",
        "Karolína",
        "Katarína",
        "Klára",
        "Klaudia",
        "Kornélia",
        "Kristína",
        "Kvetoslava",
        "Ladislava",
        "Laura",
        "Lea",
        "Lenka",
        "Leopolda",
        "Lesana",
        "Lesia",
        "Liana",
        "Libuša",
        "Liliana",
        "Linda",
        "Lívia",
        "Lolita",
        "Ľubica",
        "Ľubomíra",
        "Ľuboslava",
        "Lucia",
        "Ľudmila",
        "Ľudomila",
        "Lujza",
        "Luna",
        "Lýdia",
        "Magdaléna",
        "Malvína",
        "Marcela",
        "Margaréta",
        "Margita",
        "Mária",
        "Marianna",
        "Marika",
        "Marína",
        "Marta",
        "Martina",
        "Matilda",
        "Melánia",
        "Michaela",
        "Milada",
        "Milena",
        "Milica",
        "Miloslava",
        "Milota",
        "Miriama",
        "Miroslava",
        "Monika",
        "Nadežda",
        "Natália",
        "Nataša",
        "Nikola",
        "Nina",
        "Nora",
        "Oľga",
        "Olívia",
        "Olympia",
        "Otília",
        "Oxana",
        "Patrícia",
        "Paulína",
        "Perla",
        "Petra",
        "Petrana",
        "Petronela",
        "Radoslava",
        "Radovana",
        "Rebeka",
        "Regína",
        "Renáta",
        "Romana",
        "Rozália",
        "Ružena",
        "Sabína",
        "Sandra",
        "Sára",
        "Saskia",
        "Sidónia",
        "Silvia",
        "Simona",
        "Sláva",
        "Slávka",
        "Slavomíra",
        "Sofia",
        "Soňa",
        "Stanislava",
        "Stela",
        "Svetlana",
        "Šarlota",
        "Štefánia",
        "Tamara",
        "Tatiana",
        "Terézia",
        "Theodora",
        "Timea",
        "Uršuľa",
        "Valentína",
        "Valéria",
        "Vanda",
        "Vanesa",
        "Veronika",
        "Viera",
        "Vieroslava",
        "Viktória",
        "Vilma",
        "Viola",
        "Vladimíra",
        "Vlasta",
        "Xénia",
        "Zara",
        "Zdenka",
        "Zina",
        "Zita",
        "Zlatica",
        "Zoja",
        "Zora",
        "Zuzana",
        "Žaneta",
        "Želmíra",
        "Žofia",
    )

    first_names = first_names_male + first_names_female

    # Last names from
    # https://sk.wikipedia.org/wiki/Kategória:Priezviská_na_Slovensku

    last_names_male = (
        "Abrahám",
        "Adam",
        "Adamec",
        "Achberger",
        "Andráš",
        "Andrašík",
        "Andrášik",
        "Antal",
        "Babjak",
        "Babka",
        "Bača",
        "Bahna",
        "Bahno",
        "Baláž",
        "Bán",
        "Baňa",
        "Baňas",
        "Baňo",
        "Baran",
        "Baranka",
        "Barbora",
        "Barok",
        "Bárta",
        "Bartoš",
        "Bartošík",
        "Bartovič",
        "Baša",
        "Baška",
        "Baško",
        "Bašo",
        "Bednár",
        "Bednárik",
        "Beka",
        "Beňa",
        "Beňo",
        "Beňuš",
        "Beňuška",
        "Bernard",
        "Bernolák",
        "Bezák",
        "Bicek",
        "Bielik",
        "Bilek",
        "Bilka",
        "Bilko",
        "Blaha",
        "Blaho",
        "Blažej",
        "Bobula",
        "Bobuľa",
        "Bondra",
        "Borek",
        "Borík",
        "Bórik",
        "Borik",
        "Boris",
        "Boriš",
        "Borka",
        "Borko",
        "Borovský",
        "Borový",
        "Bosák",
        "Boška",
        "Bošňák",
        "Bôrik",
        "Brezina",
        "Brunovský",
        "Bruška",
        "Bruško",
        "Bukovský",
        "Capek",
        "Capka",
        "Capko",
        "Centek",
        "Cesnak",
        "Cibula",
        "Cibuľa",
        "Cibulka",
        "Cibuľka",
        "Cigán",
        "Cigáň",
        "Cigánik",
        "Cíger",
        "Cyprich",
        "Čapka",
        "Čapko",
        "Čapkovič",
        "Čapla",
        "Čapľa",
        "Čaplovič",
        "Čarnogurský",
        "Čekovský",
        "Černík",
        "Černoch",
        "Červeň",
        "Červený",
        "Česnek",
        "Čiernik",
        "Čierny",
        "Čobrda",
        "Ďaďo",
        "Dán",
        "Daňa",
        "Dancák",
        "Dančák",
        "Dančiak",
        "Danek",
        "Danko",
        "Daňko",
        "Dano",
        "Daňo",
        "Dávid",
        "Deák",
        "Debnár",
        "Dej",
        "Dekýš",
        "Devečka",
        "Dobšinský",
        "Dočolomanský",
        "Doležal",
        "Dora",
        "Dorka",
        "Dorko",
        "Ďorko",
        "Dorot",
        "Dorota",
        "Drajna",
        "Drobný",
        "Droppa",
        "Drotár",
        "Dubnička",
        "Dubovský",
        "Duda",
        "Dudek",
        "Dudik",
        "Dudík",
        "Dudka",
        "Dudko",
        "Dudo",
        "Dudok",
        "Dula",
        "Dulla",
        "Ďurča",
        "Ďurčo",
        "Durdík",
        "Ďurek",
        "Ďurica",
        "Ďuriš",
        "Ďurka",
        "Ďurko",
        "Ďurkovič",
        "Ďurkovský",
        "Ďuro",
        "Ďurovič",
        "Ďurovský",
        "Dusík",
        "Duska",
        "Dusko",
        "Duša",
        "Dušek",
        "Dušička",
        "Duška",
        "Dúška",
        "Duško",
        "Ďuško",
        "Dutka",
        "Ďutka",
        "Dutko",
        "Dvonč",
        "Dvorský",
        "Dzurjanin",
        "Fabian",
        "Fabián",
        "Fabuš",
        "Fajnor",
        "Farkašovský",
        "Fašiang",
        "Fedor",
        "Feldek",
        "Fico",
        "Filc",
        "Filip",
        "Filipek",
        "Filípek",
        "Filipko",
        "Finka",
        "Fogaš",
        "Forgáč",
        "Fraňa",
        "Franek",
        "Franka",
        "Franko",
        "Frano",
        "Fraňo",
        "Frimer",
        "Ftáčnik",
        "Ftorek",
        "Gál",
        "Galik",
        "Galík",
        "Gálik",
        "Gašpar",
        "Gašparovič",
        "Gavalier",
        "Gavenda",
        "Gavorník",
        "Gažo",
        "Gocník",
        "Gregor",
        "Greguš",
        "Grňa",
        "Grznár",
        "Hablák",
        "Habšuda",
        "Hagara",
        "Halák",
        "Haluška",
        "Hanak",
        "Hanák",
        "Haňak",
        "Haňák",
        "Hanek",
        "Hanka",
        "Hanko",
        "Hanus",
        "Hanusek",
        "Hanúsek",
        "Hanuska",
        "Hanúska",
        "Hanuška",
        "Hanzel",
        "Harvan",
        "Harvánek",
        "Haščák",
        "Havran",
        "Havranka",
        "Havranko",
        "Hečko",
        "Heretik",
        "Hilmer",
        "Hladký",
        "Hlaváč",
        "Hlaváčik",
        "Hliník",
        "Hlinka",
        "Hodža",
        "Holčík",
        "Holič",
        "Holiček",
        "Holíček",
        "Holička",
        "Holík",
        "Holub",
        "Holubec",
        "Holubek",
        "Holúbek",
        "Holubka",
        "Holubko",
        "Holúbok",
        "Holuby",
        "Hošták",
        "Hošťák",
        "Hoza",
        "Hraško",
        "Hrdlička",
        "Hrianka",
        "Hric",
        "Hrmo",
        "Hroboň",
        "Hruška",
        "Hrušovský",
        "Huba",
        "Hučko",
        "Hudáček",
        "Hudec",
        "Hupka",
        "Hus",
        "Hús",
        "Husák",
        "Husár",
        "Huska",
        "Húska",
        "Husťák",
        "Hušek",
        "Húšek",
        "Huška",
        "Húška",
        "Hušo",
        "Hušták",
        "Hušťák",
        "Hutta",
        "Chalupka",
        "Chovan",
        "Chovanec",
        "Chrappa",
        "Chudík",
        "Ihnačák",
        "Imrich",
        "Ivaška",
        "Jakub",
        "Jakuba",
        "Jakubček",
        "Jakubčík",
        "Jakubčo",
        "Jakubec",
        "Jakubek",
        "Jakúbek",
        "Jakubík",
        "Jakubka",
        "Jakubko",
        "Jalovec",
        "Ján",
        "Janák",
        "Janča",
        "Janček",
        "Jančo",
        "Janeček",
        "Janečka",
        "Janečko",
        "Janek",
        "Janík",
        "Janka",
        "Janko",
        "Jano",
        "Jáno",
        "Janočko",
        "Janoš",
        "Jánoš",
        "Janošek",
        "Jánošík",
        "Janoška",
        "Jánoška",
        "Janoško",
        "Jánoško",
        "Janošo",
        "Jantošovič",
        "Jedľovský",
        "Jendek",
        "Jonata",
        "Junas",
        "Junás",
        "Junáš",
        "Juráš",
        "Jurča",
        "Jurčina",
        "Jurčo",
        "Jurek",
        "Jurík",
        "Jurina",
        "Jurka",
        "Jurko",
        "Jurkovič",
        "Kaliňák",
        "Kaliský",
        "Kaniak",
        "Kára",
        "Karul",
        "Kavasch",
        "Kazik",
        "Kazík",
        "Kázik",
        "Kello",
        "Kelo",
        "Kendra",
        "Keníž",
        "Ker",
        "Kisel",
        "Kiseľ",
        "Kisela",
        "Kiseľa",
        "Klapač",
        "Klapáč",
        "Klapka",
        "Klapko",
        "Klepáč",
        "Klima",
        "Klimáček",
        "Klimek",
        "Klimko",
        "Klimkovič",
        "Klimo",
        "Kližan",
        "Klokoč",
        "Klopačka",
        "Kniha",
        "Kobielsky",
        "Kobielský",
        "Koc",
        "Kocák",
        "Kocian",
        "Kocka",
        "Kocur",
        "Kocúr",
        "Kocúrik",
        "Koč",
        "Kočiš",
        "Kočka",
        "Kočko",
        "Kolár",
        "Kolesár",
        "Kollár",
        "Kolnik",
        "Kolník",
        "Konopka",
        "Konôpka",
        "Konrád",
        "Kopa",
        "Koppa",
        "Korčák",
        "Korec",
        "Kostka",
        "Kostrec",
        "Kotian",
        "Kotulič",
        "Kouba",
        "Kováč",
        "Kováčik",
        "Kovaľ",
        "Kovalčík",
        "Koza",
        "Kozáček",
        "Kozáčik",
        "Kozák",
        "Kozmon",
        "Kôstka",
        "Krajči",
        "Krajčí",
        "Krajčír",
        "Krajčo",
        "Krajčovič",
        "Krajíček",
        "Král",
        "Kráľ",
        "Králik",
        "Kráľovič",
        "Kramár",
        "Krasnohorský",
        "Krásnohorský",
        "Krištof",
        "Kríž",
        "Križan",
        "Krížik",
        "Kropilák",
        "Kroták",
        "Krupa",
        "Krúpa",
        "Kuba",
        "Kubec",
        "Kubek",
        "Kúbek",
        "Kubiček",
        "Kubíček",
        "Kubička",
        "Kubičko",
        "Kubík",
        "Kubko",
        "Kubo",
        "Kučera",
        "Kuffa",
        "Kuna",
        "Kunda",
        "Kunka",
        "Kupa",
        "Kupka",
        "Kupko",
        "Kuruc",
        "Kus",
        "Kús",
        "Kuska",
        "Kúska",
        "Kúš",
        "Kuša",
        "Kuška",
        "Kušnír",
        "Kušo",
        "Kysel",
        "Kyseľ",
        "Kysela",
        "Kyseľa",
        "Labuda",
        "Laca",
        "Lacka",
        "Lacko",
        "Laco",
        "Lajčák",
        "Lajčiak",
        "Lamač",
        "Lepšík",
        "Lesný",
        "Leto",
        "Lipa",
        "Lipka",
        "Lipko",
        "Lipták",
        "Lisický",
        "Liška",
        "Líška",
        "Litva",
        "Loj",
        "Loja",
        "Lojek",
        "Lojka",
        "Lojko",
        "Lubina",
        "Lukáč",
        "Lukša",
        "Lupták",
        "Ľupták",
        "Mác",
        "Maca",
        "Macek",
        "Macka",
        "Mácka",
        "Macko",
        "Mackovič",
        "Maco",
        "Maček",
        "Máček",
        "Máčik",
        "Mačko",
        "Mačkovič",
        "Mačo",
        "Madar",
        "Maďar",
        "Maďár",
        "Madara",
        "Maďara",
        "Maďarič",
        "Madej",
        "Magál",
        "Mach",
        "Macháček",
        "Machata",
        "Machnáč",
        "Macho",
        "Majdan",
        "Majdán",
        "Majeský",
        "Mak",
        "Mako",
        "Makovec",
        "Makovica",
        "Makovický",
        "Makový",
        "Malachovský",
        "Malina",
        "Malíšek",
        "Mamojka",
        "Marcin",
        "Marcinek",
        "Marcinko",
        "Mareček",
        "Marek",
        "Margita",
        "Marián",
        "Marko",
        "Markoš",
        "Markovič",
        "Markuš",
        "Maroš",
        "Marta",
        "Martáň",
        "Marťan",
        "Martin",
        "Martinák",
        "Martinček",
        "Martinec",
        "Martinek",
        "Martinka",
        "Martinko",
        "Martinkovič",
        "Martinovič",
        "Masár",
        "Masarik",
        "Masarík",
        "Masárik",
        "Masaryk",
        "Maslo",
        "Masný",
        "Matiaško",
        "Mauréry",
        "Mečiar",
        "Medvecký",
        "Medveď",
        "Melichar",
        "Melichár",
        "Melichárek",
        "Melicher",
        "Melicherčík",
        "Melicherík",
        "Menšík",
        "Mihalik",
        "Mihalík",
        "Mihálik",
        "Michalech",
        "Michálech",
        "Michalek",
        "Michálek",
        "Michalica",
        "Michalík",
        "Michálik",
        "Michalka",
        "Michalko",
        "Miklík",
        "Mikloš",
        "Mikloško",
        "Mikulák",
        "Mikulík",
        "Mikuš",
        "Mikúš",
        "Milota",
        "Mináč",
        "Mistrík",
        "Mišík",
        "Mlynár",
        "Mlynárik",
        "Mojžiš",
        "Mokroš",
        "Mora",
        "Moravčík",
        "Moravec",
        "Moric",
        "Móric",
        "Motyčka",
        "Motyka",
        "Mraz",
        "Mráz",
        "Mrazek",
        "Mrázik",
        "Mrázka",
        "Mrazko",
        "Mrázko",
        "Mucha",
        "Murgaš",
        "Murin",
        "Murín",
        "Mydlo",
        "Nálepka",
        "Navara",
        "Nemec",
        "Nitra",
        "Nižnanský",
        "Nižnánsky",
        "Nižňanský",
        "Nota",
        "Novák",
        "Novota",
        "Novotný",
        "Nôta",
        "Obšut",
        "Očenáš",
        "Ondráš",
        "Ondrášek",
        "Ondrašík",
        "Ondrášik",
        "Ondrejka",
        "Ondrejko",
        "Ondrejkovič",
        "Ondrus",
        "Ondrusek",
        "Ondruš",
        "Ondrušek",
        "Ondrúšek",
        "Ondruška",
        "Ondruško",
        "Opluštil",
        "Orlík",
        "Otčenáš",
        "Pajer",
        "Paľa",
        "Palka",
        "Paľka",
        "Pálka",
        "Palko",
        "Paľko",
        "Palkovič",
        "Palla",
        "Pallo",
        "Palo",
        "Paľo",
        "Palov",
        "Paľov",
        "Palovčík",
        "Paľovčík",
        "Palovič",
        "Paluch",
        "Palúch",
        "Paľuch",
        "Paľúch",
        "Pašek",
        "Paška",
        "Paško",
        "Paúk",
        "Pauko",
        "Pauliček",
        "Paulíček",
        "Paulik",
        "Paulík",
        "Paulovič",
        "Pavel",
        "Pavelek",
        "Pavelka",
        "Pavelko",
        "Pavka",
        "Pavko",
        "Pavliček",
        "Pavlíček",
        "Pavličko",
        "Pavlik",
        "Pavlík",
        "Pavlikovský",
        "Pavlo",
        "Pavlov",
        "Pavlovič",
        "Pavol",
        "Pavuk",
        "Pavúk",
        "Pelikán",
        "Peška",
        "Peško",
        "Petrovický",
        "Petruška",
        "Pika",
        "Pišút",
        "Plaucha",
        "Plavec",
        "Plavucha",
        "Plekanec",
        "Plešivec",
        "Plch",
        "Podhradský",
        "Podkonický",
        "Polák",
        "Poliak",
        "Pollák",
        "Porubjak",
        "Pukluš",
        "Pupák",
        "Puška",
        "Puškár",
        "Rak",
        "Rakovan",
        "Repiský",
        "Riha",
        "Richard",
        "Richtár",
        "Riša",
        "Riška",
        "Riško",
        "Rišo",
        "Roháč",
        "Roháček",
        "Roháčik",
        "Rojek",
        "Rojka",
        "Rojko",
        "Róm",
        "Romančík",
        "Rosa",
        "Rosina",
        "Rubík",
        "Rúfus",
        "Rus",
        "Rusko",
        "Rusnak",
        "Rusnák",
        "Rusňak",
        "Rusňák",
        "Ružička",
        "Rybár",
        "Rybárik",
        "Rybníček",
        "Rys",
        "Rýs",
        "Ryš",
        "Ryša",
        "Ryška",
        "Ryško",
        "Salaj",
        "Salajka",
        "Sameľ",
        "Sámel",
        "Sámeľ",
        "Samel",
        "Samson",
        "Samuel",
        "Sanitra",
        "Seč",
        "Sedliak",
        "Sekera",
        "Selecký",
        "Senko",
        "Sidor",
        "Simon",
        "Sklár",
        "Sklenár",
        "Sklenárik",
        "Sklenica",
        "Sklenička",
        "Sklenka",
        "Skokan",
        "Skutecký",
        "Slašťan",
        "Slivka",
        "Slivko",
        "Sloboda",
        "Slobodník",
        "Slota",
        "Slovák",
        "Slovinec",
        "Smrek",
        "Sokol",
        "Solej",
        "Starosta",
        "Stodola",
        "Straka",
        "Strnisko",
        "Strýček",
        "Svrbík",
        "Sýkora",
        "Šalajka",
        "Šarkan",
        "Šebo",
        "Šidlo",
        "Šima",
        "Šimek",
        "Šimík",
        "Šimka",
        "Šimko",
        "Šimo",
        "Šimon",
        "Šimončič",
        "Šimonovič",
        "Škantár",
        "Škoviera",
        "Škriniar",
        "Škultéty",
        "Šlahor",
        "Šlivka",
        "Šmajda",
        "Šoltés",
        "Šoltýs",
        "Špilár",
        "Šťastný",
        "Štefan",
        "Štefánek",
        "Štefánik",
        "Štefanka",
        "Štefanko",
        "Štefanovič",
        "Štefunko",
        "Štrba",
        "Šulc",
        "Šuňavec",
        "Šurka",
        "Šváb",
        "Šváby",
        "Švehla",
        "Talpaš",
        "Ťapák",
        "Tatar",
        "Tatár",
        "Tatarka",
        "Tatárka",
        "Ťažký",
        "Thomka",
        "Timek",
        "Timka",
        "Timko",
        "Tirpák",
        "Toman",
        "Tomaň",
        "Tomana",
        "Tomanek",
        "Tománek",
        "Tomaník",
        "Tomáš",
        "Tomašek",
        "Tomášek",
        "Tomášik",
        "Tomaška",
        "Tomáška",
        "Tomaško",
        "Tomáško",
        "Tomeček",
        "Tomečka",
        "Tomečko",
        "Tomek",
        "Tomka",
        "Tomko",
        "Topoľský",
        "Trlík",
        "Truben",
        "Tupý",
        "Turčok",
        "Uram",
        "Urblík",
        "Vajcik",
        "Vajcík",
        "Vajda",
        "Valach",
        "Valachovič",
        "Valent",
        "Valenta",
        "Valentín",
        "Valíček",
        "Valušek",
        "Valuška",
        "Vanek",
        "Vaska",
        "Váska",
        "Vasko",
        "Vašek",
        "Vaška",
        "Vaško",
        "Vavrinec",
        "Velďák",
        "Vesel",
        "Veterník",
        "Vicen",
        "Vicena",
        "Višňovský",
        "Vlach",
        "Vlachovič",
        "Vlk",
        "Vlna",
        "Vlnka",
        "Vojtek",
        "Vojtka",
        "Vojtko",
        "Vozar",
        "Vozár",
        "Vrabec",
        "Vrablec",
        "Vydarený",
        "Záborský",
        "Zachar",
        "Zachara",
        "Zachariaš",
        "Zachariáš",
        "Zajac",
        "Zigo",
        "Zima",
        "Zimka",
        "Zubek",
        "Zúbek",
        "Zubka",
        "Zubko",
        "Zubrík",
        "Zúbrik",
        "Železník",
        "Žigmund",
        "Žigo",
    )

    # Last names (adjusted manually) from
    # https://sk.wikipedia.org/wiki/Kategória:Priezviská_na_Slovensku

    last_names_female = (
        "Abrahámová",
        "Adamová",
        "Adamecová",
        "Achbergerová",
        "Andrášová",
        "Andrašíková",
        "Andrášiková",
        "Antalová",
        "Babjaková",
        "Bahnová",
        "Balážová",
        "Bánová",
        "Baňasová",
        "Baranová",
        "Baranková",
        "Baroková",
        "Bártová",
        "Bartošová",
        "Bartošíková",
        "Bartovičová",
        "Bašová",
        "Bašková",
        "Bednárová",
        "Bednáriková",
        "Beňová",
        "Beňušová",
        "Beňušková",
        "Bernardová",
        "Bernoláková",
        "Bezáková",
        "Biceková",
        "Bieliková",
        "Bileková",
        "Bilková",
        "Blahová",
        "Blažejová",
        "Bobulová",
        "Bobuľová",
        "Bondrová",
        "Boreková",
        "Boríková",
        "Bóriková",
        "Boriková",
        "Borisová",
        "Borišová",
        "Borková",
        "Bosáková",
        "Bošňáková",
        "Bôriková",
        "Brezinová",
        "Brunovská",
        "Brušková",
        "Bukovská",
        "Capeková",
        "Capková",
        "Centeková",
        "Cesnaková",
        "Cibulová",
        "Cibuľková",
        "Cigánová",
        "Cigáňová",
        "Cigániková",
        "Cígerová",
        "Cyprichová",
        "Čapková",
        "Čapkovičová",
        "Čaplovičová",
        "Čarnogurská",
        "Čekovská",
        "Černíková",
        "Černochová",
        "Červeňová",
        "Česneková",
        "Čierniková",
        "Čiernaová",
        "Čobrdová",
        "Ďaďová",
        "Dánová",
        "Daňová",
        "Dancáková",
        "Dančáková",
        "Dančiaková",
        "Daneková",
        "Danková",
        "Daňková",
        "Danová",
        "Dávidová",
        "Deáková",
        "Debnárová",
        "Dekýšová",
        "Devečková",
        "Dobšinská",
        "Dočolomanská",
        "Doležalová",
        "Dorová",
        "Dorková",
        "Ďorková",
        "Dorotová",
        "Drajnová",
        "Drobná",
        "Droppová",
        "Drotárová",
        "Dubovská",
        "Dudová",
        "Dudeková",
        "Dudíková",
        "Dudková",
        "Dudoková",
        "Dulová",
        "Dullová",
        "Ďurčová",
        "Durdíková",
        "Ďureková",
        "Ďuricová",
        "Ďurišová",
        "Ďurkovičová",
        "Ďurkovská",
        "Ďurová",
        "Ďurovičová",
        "Ďurovskýová",
        "Dusíková",
        "Dusková",
        "Dušová",
        "Dušeková",
        "Dušičková",
        "Dutková",
        "Ďutková",
        "Dvončová",
        "Dvorská",
        "Dzurjaninová",
        "Fabianová",
        "Fabiánová",
        "Fabušová",
        "Fajnorová",
        "Farkašovská",
        "Fašiangová",
        "Fedorová",
        "Feldeková",
        "Ficová",
        "Filcová",
        "Filipová",
        "Filipeková",
        "Filípeková",
        "Filipková",
        "Finková",
        "Fogašová",
        "Forgáčová",
        "Fraňová",
        "Franeková",
        "Franková",
        "Franová",
        "Fraňvá",
        "Frimerová",
        "Ftáčniková",
        "Ftoreková",
        "Gálová",
        "Galiková",
        "Galíková",
        "Gáliková",
        "Gašparová",
        "Gašparovičová",
        "Gavalierová",
        "Gavendová",
        "Gavorníková",
        "Gažová",
        "Gocníková",
        "Gregorová",
        "Gregušová",
        "Grňová",
        "Grznárová",
        "Habláková",
        "Habšudová",
        "Hagarová",
        "Haláková",
        "Halušková",
        "Hanaková",
        "Hanáková",
        "Haňaková",
        "Haňáková",
        "Haneková",
        "Hanková",
        "Hanusová",
        "Hanuseková",
        "Hanúseková",
        "Hanusková",
        "Hanúsková",
        "Hanušková",
        "Hanzelová",
        "Harvanová",
        "Harváneková",
        "Haščáková",
        "Havranová",
        "Havranková",
        "Hečková",
        "Heretiková",
        "Hilmerová",
        "Hladká",
        "Hlaváčová",
        "Hlaváčiková",
        "Hliníková",
        "Hlinková",
        "Hodžová",
        "Holčíková",
        "Holičová",
        "Holičeková",
        "Holíčeková",
        "Holičková",
        "Holíková",
        "Holubová",
        "Holubecová",
        "Holubeková",
        "Holúbeková",
        "Holubková",
        "Hoštáková",
        "Hošťáková",
        "Hozová",
        "Hrašková",
        "Hrdličková",
        "Hrianková",
        "Hricová",
        "Hrmová",
        "Hroboňová",
        "Hrušková",
        "Hrušovská",
        "Hubová",
        "Hučková",
        "Hudáčeková",
        "Hudecová",
        "Hupková",
        "Husová",
        "Húsová",
        "Husáková",
        "Husárová",
        "Húsková",
        "Husťáková",
        "Hušeková",
        "Húšeková",
        "Hušová",
        "Huštáková",
        "Hušťáková",
        "Huttová",
        "Chalupková",
        "Chovanová",
        "Chovanecová",
        "Chrappová",
        "Chudíková",
        "Ihnačáková",
        "Imrichová",
        "Ivašková",
        "Jakubová",
        "Jakubčeková",
        "Jakubčíková",
        "Jakubčová",
        "Jakubecová",
        "Jakubeková",
        "Jakúbeková",
        "Jakubíková",
        "Jakubková",
        "Jalovecová",
        "Jánová",
        "Janáková",
        "Jančová",
        "Jančeková",
        "Janečeková",
        "Janečková",
        "Janeková",
        "Janíková",
        "Janková",
        "Janová",
        "Jánová",
        "Janočková",
        "Janošová",
        "Jánošová",
        "Janošeková",
        "Jánošíková",
        "Janošková",
        "Jánošková",
        "Janošová",
        "Jantošovičová",
        "Jedľovská",
        "Jendeková",
        "Jonatová",
        "Junasová",
        "Junásová",
        "Junášová",
        "Jurášová",
        "Jurčová",
        "Jurčinová",
        "Jureková",
        "Juríková",
        "Jurinová",
        "Jurková",
        "Jurkovičová",
        "Kaliňáková",
        "Kaliská",
        "Kaniaková",
        "Kárová",
        "Karulová",
        "Kavaschová",
        "Kaziková",
        "Kazíková",
        "Káziková",
        "Kellová",
        "Kelová",
        "Kendrová",
        "Kenížová",
        "Kerová",
        "Kiselová",
        "Kiseľová",
        "Klapačová",
        "Klapáčová",
        "Klapková",
        "Klepáčová",
        "Klimová",
        "Klimáčeková",
        "Klimeková",
        "Klimková",
        "Klimkovičová",
        "Kližanová",
        "Klokočová",
        "Klopačková",
        "Knihová",
        "Kobielská",
        "Kocová",
        "Kocáková",
        "Kocianová",
        "Kocková",
        "Kocúrová",
        "Kocúriková",
        "Kočová",
        "Kočišová",
        "Kočková",
        "Kolárová",
        "Kolesárová",
        "Kollárová",
        "Kolniková",
        "Kolníková",
        "Konopková",
        "Konôpková",
        "Konrádová",
        "Kopová",
        "Koppová",
        "Korčáková",
        "Korecová",
        "Kostková",
        "Kostrecová",
        "Kotianová",
        "Kotuličová",
        "Koubová",
        "Kováčová",
        "Kováčiková",
        "Kovaľová",
        "Kovalčíková",
        "Kozová",
        "Kozáčeková",
        "Kozáčiková",
        "Kozáková",
        "Kozmonová",
        "Kôstková",
        "Krajčiová",
        "Krajčíová",
        "Krajčírová",
        "Krajčová",
        "Krajčovičová",
        "Krajíčeková",
        "Králová",
        "Kráľová",
        "Králiková",
        "Kráľovičová",
        "Kramárová",
        "Krasnohorská",
        "Krásnohorská",
        "Krištofová",
        "Krížová",
        "Križanová",
        "Krížiková",
        "Kropiláková",
        "Krotáková",
        "Krupová",
        "Krúpová",
        "Kubová",
        "Kubecová",
        "Kubeková",
        "Kúbeková",
        "Kubičeková",
        "Kubíčeková",
        "Kubičková",
        "Kubíková",
        "Kubková",
        "Kučerová",
        "Kuffová",
        "Kunová",
        "Kundová",
        "Kunková",
        "Kupová",
        "Kupková",
        "Kurucová",
        "Kusová",
        "Kúsová",
        "Kusková",
        "Kúsková",
        "Kúšová",
        "Kušová",
        "Kušková",
        "Kušnírová",
        "Kyselová",
        "Kyseľová",
        "Labudová",
        "Lacová",
        "Lacková",
        "Lajčáková",
        "Lajčiaková",
        "Lamačová",
        "Lepšíková",
        "Lesná",
        "Letová",
        "Lipová",
        "Lipková",
        "Liptáková",
        "Lisická",
        "Lišková",
        "Líšková",
        "Litvová",
        "Lojová",
        "Lojeková",
        "Lojková",
        "Lubinová",
        "Lukáčová",
        "Lukšová",
        "Luptáková",
        "Ľuptáková",
        "Mácová",
        "Macová",
        "Maceková",
        "Macková",
        "Mácková",
        "Mackovičová",
        "Mačeková",
        "Máčeková",
        "Máčiková",
        "Mačková",
        "Mačkovičová",
        "Madarová",
        "Maďarová",
        "Maďárová",
        "Maďaričová",
        "Madejová",
        "Magálová",
        "Machová",
        "Macháčeková",
        "Machatová",
        "Machnáčová",
        "Machová",
        "Majdanová",
        "Majdánová",
        "Majeská",
        "Maková",
        "Makovecová",
        "Makovická",
        "Maková",
        "Malachovská",
        "Malinová",
        "Malíšeková",
        "Mamojková",
        "Marcinová",
        "Marcineková",
        "Marcinková",
        "Marečeková",
        "Mareková",
        "Margitová",
        "Mariánová",
        "Marková",
        "Markošová",
        "Markovičová",
        "Markušová",
        "Marošová",
        "Martová",
        "Martáňová",
        "Marťanová",
        "Martinová",
        "Martináková",
        "Martinčeková",
        "Martinecová",
        "Martineková",
        "Martinková",
        "Martinkovičová",
        "Martinovičová",
        "Masárová",
        "Masariková",
        "Masaríková",
        "Masáriková",
        "Masaryková",
        "Maslová",
        "Masná",
        "Matiašková",
        "Mauréryová",
        "Mečiarová",
        "Medvecká",
        "Medveďová",
        "Melicharová",
        "Melichárová",
        "Melicháreková",
        "Melicherová",
        "Melicherčíková",
        "Melicheríková",
        "Menšíková",
        "Mihaliková",
        "Mihalíková",
        "Miháliková",
        "Michalechová",
        "Michálechová",
        "Michaleková",
        "Micháleková",
        "Michalicová",
        "Michalíková",
        "Micháliková",
        "Michalková",
        "Miklíková",
        "Miklošová",
        "Miklošková",
        "Mikuláková",
        "Mikulíková",
        "Mikušová",
        "Mikúšová",
        "Milotová",
        "Mináčová",
        "Mistríková",
        "Mišíková",
        "Mlynárová",
        "Mlynáriková",
        "Mojžišová",
        "Mokrošová",
        "Morová",
        "Moravčíková",
        "Moravecová",
        "Moricová",
        "Móricová",
        "Motyčková",
        "Motyková",
        "Mrazová",
        "Mrázová",
        "Mrazeková",
        "Mráziková",
        "Mrázková",
        "Mrazková",
        "Muchová",
        "Murgašová",
        "Murinová",
        "Murínová",
        "Mydlová",
        "Nálepková",
        "Navarová",
        "Nemcová",
        "Nitrová",
        "Nižnanská",
        "Nižnánská",
        "Nižňanská",
        "Notová",
        "Nováková",
        "Novotná",
        "Nôtová",
        "Obšutová",
        "Očenášová",
        "Ondrášová",
        "Ondrášeková",
        "Ondrašíková",
        "Ondrášiková",
        "Ondrejková",
        "Ondrejkovičová",
        "Ondruseková",
        "Ondrušová",
        "Ondrušeková",
        "Ondrúšeková",
        "Ondrušková",
        "Opluštilová",
        "Orlíková",
        "Otčenášová",
        "Pajerová",
        "Paľová",
        "Palková",
        "Paľková",
        "Palkovičová",
        "Pallová",
        "Palová",
        "Palovová",
        "Paľovová",
        "Palovčíková",
        "Paľovčíková",
        "Palovičová",
        "Paluchová",
        "Palúchová",
        "Paľuchová",
        "Paľúchová",
        "Pašeková",
        "Pašková",
        "Paúková",
        "Pauková",
        "Pauličeková",
        "Paulíčeková",
        "Pauliková",
        "Paulíková",
        "Paulovičová",
        "Pavelová",
        "Paveleková",
        "Pavelková",
        "Pavková",
        "Pavličeková",
        "Pavlíčeková",
        "Pavliková",
        "Pavlíková",
        "Pavlikovská",
        "Pavlová",
        "Pavlovová",
        "Pavlovičová",
        "Pavolová",
        "Pavúková",
        "Pelikánová",
        "Pešková",
        "Petrovická",
        "Petrušková",
        "Piková",
        "Pišútová",
        "Plauchová",
        "Plavecová",
        "Plavuchová",
        "Plekancová",
        "Plešivcová",
        "Plchová",
        "Podhradská",
        "Podkonická",
        "Poláková",
        "Poliaková",
        "Polláková",
        "Porubjaková",
        "Puklušová",
        "Pupáková",
        "Pušková",
        "Puškárová",
        "Raková",
        "Rakovanová",
        "Repiská",
        "Rihová",
        "Richardová",
        "Richtárová",
        "Rišová",
        "Rišková",
        "Roháčová",
        "Roháčeková",
        "Roháčiková",
        "Rojeková",
        "Rojková",
        "Rómová",
        "Romančíková",
        "Rosová",
        "Rosinová",
        "Rubíková",
        "Rúfusová",
        "Rusová",
        "Rusková",
        "Rusnáková",
        "Rusňáková",
        "Ružičková",
        "Rybárová",
        "Rybáriková",
        "Rybníčeková",
        "Rysová",
        "Rýsová",
        "Ryšová",
        "Ryšková",
        "Salajová",
        "Salajková",
        "Sameľová",
        "Sámelová",
        "Sámeľová",
        "Samelová",
        "Samsonová",
        "Samuelová",
        "Sanitrová",
        "Sečová",
        "Sedliaková",
        "Sekerová",
        "Selecká",
        "Senková",
        "Sidorová",
        "Simonová",
        "Sklárová",
        "Sklenárová",
        "Sklenáriková",
        "Sklenicová",
        "Skleničková",
        "Sklenková",
        "Skokanová",
        "Skutecká",
        "Slašťanová",
        "Slivková",
        "Slobodová",
        "Slobodníková",
        "Slotová",
        "Slováková",
        "Slovinecová",
        "Smreková",
        "Sokolová",
        "Solejová",
        "Starostová",
        "Stodolová",
        "Straková",
        "Strnisková",
        "Strýčeková",
        "Svrbíková",
        "Sýkorová",
        "Šalajková",
        "Šarkanová",
        "Šebová",
        "Šidlová",
        "Šimová",
        "Šimeková",
        "Šimíková",
        "Šimková",
        "Šimonová",
        "Šimončičová",
        "Šimonovičová",
        "Škantárová",
        "Škovierová",
        "Škriniarová",
        "Škultétyová",
        "Šlahorová",
        "Šlivková",
        "Šmajdová",
        "Šoltésová",
        "Šoltýsová",
        "Špilárová",
        "Šťastná",
        "Štefanová",
        "Štefáneková",
        "Štefániková",
        "Štefanková",
        "Štefanovičová",
        "Štefunková",
        "Štrbová",
        "Šulcová",
        "Šuňavcová",
        "Šurková",
        "Švábová",
        "Švehlová",
        "Talpašová",
        "Ťapáková",
        "Tatarová",
        "Tatárová",
        "Tatarková",
        "Ťažká",
        "Timeková",
        "Timková",
        "Tirpáková",
        "Tomanová",
        "Tomaňová",
        "Tománeková",
        "Tomaníková",
        "Tomášová",
        "Tomašeková",
        "Tomášeková",
        "Tomášiková",
        "Tomašková",
        "Tomášková",
        "Tomečeková",
        "Tomečková",
        "Tomeková",
        "Tomková",
        "Topoľská",
        "Trlíková",
        "Trubenová",
        "Tupá",
        "Turčoková",
        "Uramová",
        "Urblíková",
        "Vajciková",
        "Vajcíková",
        "Vajdová",
        "Valachová",
        "Valachovičová",
        "Valentová",
        "Valentínová",
        "Valíčeková",
        "Valušeková",
        "Valušková",
        "Vaneková",
        "Vasková",
        "Vásková",
        "Vašeková",
        "Vašková",
        "Vavrinecová",
        "Velďáková",
        "Veselová",
        "Veterníková",
        "Vicenová",
        "Višňovská",
        "Vlachová",
        "Vlachovičová",
        "Vlková",
        "Vlnová",
        "Vlnková",
        "Vojteková",
        "Vojtková",
        "Vozárová",
        "Vrabcová",
        "Vrablecová",
        "Vydarená",
        "Záborská",
        "Zacharová",
        "Zachariašová",
        "Zachariášová",
        "Zajacová",
        "Zigová",
        "Zimová",
        "Zimková",
        "Zubeková",
        "Zúbeková",
        "Zubková",
        "Zubríková",
        "Zúbriková",
        "Železníková",
        "Žigmundová",
        "Žigová",
    )

    last_names = last_names_male + last_names_female

    # Degrees from
    degrees = ("JUDr.", "Ing.", "Bc.", "Mgr.", "MUDr.", "RNDr.", "Ing. arch.", "MVDr.", "PhDr.")

    prefixes_male = ("pán",) + degrees

    prefixes_female = ("pani", "slečna") + degrees

    suffixes = (
        "CSc.",
        "DiS.",
        "Ph.D.",
        "Th.D.",
        "DSc.",
    )



from ..es import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = (
        "{{first_name_female}} {{last_name}} {{last_name}}",
        "{{first_name_female}} {{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{first_name_female}} {{last_name}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{prefix_female}} {{first_name_female}} {{last_name}}",
    )

    formats_male = (
        "{{first_name_male}} {{last_name}} {{last_name}}",
        "{{first_name_male}} {{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{first_name_male}} {{last_name}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{prefix_male}} {{first_name_male}} {{last_name}}",
    )

    formats = formats_male + formats_female

    first_names_male = (
        "Abel",
        "Abelardo",
        "Abraham",
        "Adalberto",
        "Adán",
        "Agustín",
        "Alberto",
        "Aldo",
        "Alejandro",
        "Alfonso",
        "Alfredo",
        "Alonso",
        "Alvaro",
        "Amador",
        "Andrés",
        "Ángel",
        "Antonio",
        "Armando",
        "Arturo",
        "Augusto",
        "Aurelio",
        "Benito",
        "Benjamín",
        "Bernabé",
        "Bernardo",
        "Bruno",
        "Camilo",
        "Carlos",
        "César",
        "Claudio",
        "Clemente",
        "Cornelio",
        "Cristian",
        "Cristobal",
        "Daniel",
        "Darío",
        "David",
        "Diego",
        "Eduardo",
        "Elias",
        "Eloy",
        "Emiliano",
        "Emilio",
        "Enrique",
        "Eric",
        "Ernesto",
        "Espartaco",
        "Esteban",
        "Eugenio",
        "Evelio",
        "Federico",
        "Felipe",
        "Felix",
        "Fernando",
        "Fidel",
        "Flavio",
        "Francisco",
        "Francisco Javier",
        "Gabino",
        "Gabriel",
        "Genaro",
        "Gerardo",
        "Germán",
        "Gerónimo",
        "Gilberto",
        "Gonzalo",
        "Gregorio",
        "Guillermo",
        "Gustavo",
        "Héctor",
        "Hernán",
        "Homero",
        "Horacio",
        "Hugo",
        "Humberto",
        "Ignacio",
        "Isaac",
        "Israel",
        "Iván",
        "Jacinto",
        "Jacobo",
        "Jaime",
        "Javier",
        "Jerónimo",
        "Jesús",
        "Joaquín",
        "Jonás",
        "Jorge",
        "Jorge Luis",
        "Jos",
        "José",
        "José Carlos",
        "José Eduardo",
        "José Emilio",
        "José Luis",
        "José Manuél",
        "José María",
        "Juan",
        "Juan Carlos",
        "Julio",
        "Julio César",
        "Leonardo",
        "Leonel",
        "Lorenzo",
        "Lucas",
        "Luis",
        "Luis Manuel",
        "Luis Miguel",
        "Manuel",
        "Marco Antonio",
        "Marcos",
        "Mariano",
        "Mario",
        "Martín",
        "Mateo",
        "Mauricio",
        "Mauro",
        "Maximiliano",
        "Miguel",
        "Miguel Ángel",
        "Modesto",
        "Nicolás",
        "Octavio",
        "Óliver",
        "Omar",
        "Óscar",
        "Oswaldo",
        "Pablo",
        "Pascual",
        "Patricio",
        "Pedro",
        "Porfirio",
        "Rafaél",
        "Ramiro",
        "Ramón",
        "Raúl",
        "Renato",
        "René",
        "Reynaldo",
        "Ricardo",
        "Roberto",
        "Rodolfo",
        "Rodrigo",
        "Rolando",
        "Rubén",
        "Rufino",
        "Salvador",
        "Samuel",
        "Santiago",
        "Serafín",
        "Sergio",
        "Silvano",
        "Teodoro",
        "Timoteo",
        "Tomás",
        "Uriel",
        "Vicente",
        "Víctor",
        "Wilfrido",
        "Zacarías",
        "Zeferino",
    )

    first_names_female = (
        "Abigail",
        "Abril",
        "Adela",
        "Adriana",
        "Aida",
        "Alejandra",
        "Aldonza",
        "Alicia",
        "Alma",
        "Alta  Gracia",
        "Amalia",
        "Amanda",
        "Amelia",
        "Ana",
        "Ana Luisa",
        "Ana María",
        "Anabel",
        "Andrea",
        "Anel",
        "Ángela",
        "Angélica",
        "Antonia",
        "Araceli",
        "Arcelia",
        "Ariadna",
        "Asunción",
        "Aurora",
        "Barbara",
        "Beatriz",
        "Berta",
        "Bianca",
        "Blanca",
        "Camila",
        "Caridad",
        "Carla",
        "Carlota",
        "Carmen",
        "Carolina",
        "Catalina",
        "Cecilia",
        "Celia",
        "Citlali",
        "Clara",
        "Claudia",
        "Concepción",
        "Conchita",
        "Cristal",
        "Cristina",
        "Cynthia",
        "Dalia",
        "Daniela",
        "Débora",
        "Delia",
        "Diana",
        "Dolores",
        "Dulce",
        "Dulce María",
        "Elena",
        "Elisa",
        "Eloisa",
        "Elsa",
        "Elvia",
        "Elvira",
        "Emilia",
        "Esmeralda",
        "Esparta",
        "Esperanza",
        "Estefanía",
        "Estela",
        "Estela",
        "Esther",
        "Eugenia",
        "Eva",
        "Fabiola",
        "Florencia",
        "Francisca",
        "Frida",
        "Gabriela",
        "Georgina",
        "Gloria",
        "Graciela",
        "Guadalupe",
        "Guillermina",
        "Helena",
        "Hermelinda",
        "Hilda",
        "Ilse",
        "Indira",
        "Inés",
        "Irene",
        "Irma",
        "Isabel",
        "Isabela",
        "Itzel",
        "Ivonne",
        "Jaqueline",
        "Josefina",
        "Juana",
        "Judith",
        "Julia",
        "Karina",
        "Karla",
        "Laura",
        "Leonor",
        "Leticia",
        "Lilia",
        "Liliana",
        "Linda",
        "Lorena",
        "Lourdes",
        "Lucía",
        "Luisa",
        "Luz",
        "Magdalena",
        "Marcela",
        "Margarita",
        "María",
        "María Cristina",
        "María del Carmen",
        "María Elena",
        "María Eugenia",
        "María José",
        "María Luisa",
        "María Teresa",
        "Mariana",
        "Marisela",
        "Marisol",
        "Marisol",
        "Martha",
        "Mayte",
        "Mercedes",
        "Micaela",
        "Minerva",
        "Miriam",
        "Mitzy",
        "Mónica",
        "Nadia",
        "Nancy",
        "Natalia",
        "Natividad",
        "Nayeli",
        "Nelly",
        "Noelia",
        "Noemí",
        "Norma",
        "Ofelia",
        "Olga",
        "Olivia",
        "Pamela",
        "Paola",
        "Patricia",
        "Paulina",
        "Perla",
        "Pilar",
        "Raquel",
        "Rebeca",
        "Reina",
        "Rocío",
        "Rosa",
        "Rosalia",
        "Rosario",
        "Ruby",
        "Salma",
        "Sandra",
        "Sara",
        "Sessa",
        "Silvia",
        "Socorro",
        "Sofía",
        "Soledad",
        "Sonia",
        "Susana",
        "Tania",
        "Teresa",
        "Trinidad",
        "Úrsula",
        "Vanesa",
        "Verónica",
        "Victoria",
        "Violeta",
        "Virginia",
        "Wendolin",
        "Yeni",
        "Yolanda",
        "Yuridia",
        "Zoé",
    )

    first_names = first_names_male + first_names_female

    prefixes_female = ("Sra.", "Srita.", "Dra.", "Ing.", "Lic.")
    prefixes_male = ("Sr.", "Dr.", "Ing.", "Lic.")
    last_names = (
        "Abrego",
        "Abreu",
        "Acevedo",
        "Acosta",
        "Acuña",
        "Adame",
        "Aguayo",
        "Aguilar",
        "Aguilera",
        "Aguirre",
        "Alarcón",
        "Alba",
        "Alcala",
        "Alcántar",
        "Alcaraz",
        "Alejandro",
        "Alemán",
        "Alfaro",
        "Almanza",
        "Almaraz",
        "Almonte",
        "Alonso",
        "Alonzo",
        "Altamirano",
        "Alva",
        "Alvarado",
        "Alvarez",
        "Amador",
        "Amaya",
        "Anaya",
        "Anguiano",
        "Angulo",
        "Aparicio",
        "Apodaca",
        "Aponte",
        "Aragón",
        "Aranda",
        "Arce",
        "Archuleta",
        "Arellano",
        "Arenas",
        "Arevalo",
        "Arguello",
        "Arias",
        "Armas",
        "Armendáriz",
        "Armenta",
        "Arredondo",
        "Arreola",
        "Arriaga",
        "Arroyo",
        "Arteaga",
        "Ávalos",
        "Ávila",
        "Avilés",
        "Ayala",
        "Baca",
        "Badillo",
        "Báez",
        "Baeza",
        "Bahena",
        "Balderas",
        "Ballesteros",
        "Bañuelos",
        "Barajas",
        "Barela",
        "Barragán",
        "Barraza",
        "Barrera",
        "Barreto",
        "Barrientos",
        "Barrios",
        "Batista",
        "Becerra",
        "Beltrán",
        "Benavides",
        "Benavídez",
        "Benítez",
        "Bermúdez",
        "Bernal",
        "Berríos",
        "Bétancourt",
        "Blanco",
        "Bonilla",
        "Borrego",
        "Botello",
        "Bravo",
        "Briones",
        "Briseño",
        "Brito",
        "Bueno",
        "Burgos",
        "Bustamante",
        "Bustos",
        "Caballero",
        "Cabán",
        "Cabrera",
        "Cadena",
        "Caldera",
        "Calderón",
        "Calvillo",
        "Camacho",
        "Camarillo",
        "Campos",
        "Canales",
        "Candelaria",
        "Cano",
        "Cantú",
        "Caraballo",
        "Carbajal",
        "Cardenas",
        "Cardona",
        "Carmona",
        "Carranza",
        "Carrasco",
        "Carreón",
        "Carrera",
        "Carrero",
        "Carrillo",
        "Carrión",
        "Carvajal",
        "Casanova",
        "Casares",
        "Casárez",
        "Casas",
        "Casillas",
        "Castañeda",
        "Castellanos",
        "Castillo",
        "Castro",
        "Cavazos",
        "Cazares",
        "Ceballos",
        "Cedillo",
        "Ceja",
        "Centeno",
        "Cepeda",
        "Cervantes",
        "Cervántez",
        "Chacón",
        "Chapa",
        "Chavarría",
        "Chávez",
        "Cintrón",
        "Cisneros",
        "Collado",
        "Collazo",
        "Colón",
        "Colunga",
        "Concepción",
        "Contreras",
        "Cordero",
        "Córdova",
        "Cornejo",
        "Corona",
        "Coronado",
        "Corral",
        "Corrales",
        "Correa",
        "Cortés",
        "Cortez",
        "Cotto",
        "Covarrubias",
        "Crespo",
        "Cruz",
        "Cuellar",
        "Curiel",
        "Dávila",
        "de Anda",
        "de Jesús",
        "de la Crúz",
        "de la Fuente",
        "de la Garza",
        "de la O",
        "de la Rosa",
        "de la Torre",
        "de León",
        "Delgadillo",
        "Delgado",
        "del Río",
        "del Valle",
        "Díaz",
        "Domínguez",
        "Duarte",
        "Dueñas",
        "Durán",
        "Echeverría",
        "Elizondo",
        "Enríquez",
        "Escalante",
        "Escamilla",
        "Escobar",
        "Escobedo",
        "Esparza",
        "Espinal",
        "Espino",
        "Espinosa",
        "Espinoza",
        "Esquibel",
        "Esquivel",
        "Estévez",
        "Estrada",
        "Fajardo",
        "Farías",
        "Feliciano",
        "Fernández",
        "Ferrer",
        "Fierro",
        "Figueroa",
        "Flores",
        "Flórez",
        "Fonseca",
        "Franco",
        "Frías",
        "Fuentes",
        "Gaitán",
        "Galarza",
        "Galindo",
        "Gallardo",
        "Gallegos",
        "Galván",
        "Gálvez",
        "Gamboa",
        "Gamez",
        "Gaona",
        "Garay",
        "García",
        "Garibay",
        "Garica",
        "Garrido",
        "Garza",
        "Gastélum",
        "Gaytán",
        "Gil",
        "Girón",
        "Godínez",
        "Godoy",
        "Gómez",
        "Gonzales",
        "González",
        "Gollum",
        "Gracia",
        "Granado",
        "Granados",
        "Griego",
        "Grijalva",
        "Guajardo",
        "Guardado",
        "Guerra",
        "Guerrero",
        "Guevara",
        "Guillen",
        "Gurule",
        "Gutiérrez",
        "Guzmán",
        "Haro",
        "Henríquez",
        "Heredia",
        "Hernádez",
        "Hernandes",
        "Hernández",
        "Herrera",
        "Hidalgo",
        "Hinojosa",
        "Holguín",
        "Huerta",
        "Hurtado",
        "Ibarra",
        "Iglesias",
        "Irizarry",
        "Jaime",
        "Jaimes",
        "Jáquez",
        "Jaramillo",
        "Jasso",
        "Jiménez",
        "Jimínez",
        "Juárez",
        "Jurado",
        "Laboy",
        "Lara",
        "Laureano",
        "Leal",
        "Lebrón",
        "Ledesma",
        "Leiva",
        "Lemus",
        "León",
        "Lerma",
        "Leyva",
        "Limón",
        "Linares",
        "Lira",
        "Llamas",
        "Loera",
        "Lomeli",
        "Longoria",
        "López",
        "Lovato",
        "Loya",
        "Lozada",
        "Lozano",
        "Lucero",
        "Lucio",
        "Luevano",
        "Lugo",
        "Luna",
        "Macías",
        "Madera",
        "Madrid",
        "Madrigal",
        "Maestas",
        "Magaña",
        "Malave",
        "Maldonado",
        "Manzanares",
        "Mares",
        "Marín",
        "Márquez",
        "Marrero",
        "Marroquín",
        "Martínez",
        "Mascareñas",
        "Mata",
        "Mateo",
        "Matías",
        "Matos",
        "Maya",
        "Mayorga",
        "Medina",
        "Medrano",
        "Mejía",
        "Meléndez",
        "Melgar",
        "Mena",
        "Menchaca",
        "Méndez",
        "Mendoza",
        "Menéndez",
        "Meraz",
        "Mercado",
        "Merino",
        "Mesa",
        "Meza",
        "Miramontes",
        "Miranda",
        "Mireles",
        "Mojica",
        "Molina",
        "Mondragón",
        "Monroy",
        "Montalvo",
        "Montañez",
        "Montaño",
        "Montemayor",
        "Montenegro",
        "Montero",
        "Montes",
        "Montez",
        "Montoya",
        "Mora",
        "Morales",
        "Moreno",
        "Mota",
        "Moya",
        "Munguía",
        "Muñiz",
        "Muñoz",
        "Murillo",
        "Muro",
        "Nájera",
        "Naranjo",
        "Narváez",
        "Nava",
        "Navarrete",
        "Navarro",
        "Nazario",
        "Negrete",
        "Negrón",
        "Nevárez",
        "Nieto",
        "Nieves",
        "Niño",
        "Noriega",
        "Núñez",
        "Ocampo",
        "Ocasio",
        "Ochoa",
        "Ojeda",
        "Olivares",
        "Olivárez",
        "Olivas",
        "Olivera",
        "Olivo",
        "Olmos",
        "Olvera",
        "Ontiveros",
        "Oquendo",
        "Ordóñez",
        "Orellana",
        "Ornelas",
        "Orosco",
        "Orozco",
        "Orta",
        "Ortega",
        "Ortiz",
        "Osorio",
        "Otero",
        "Ozuna",
        "Pabón",
        "Pacheco",
        "Padilla",
        "Padrón",
        "Páez",
        "Palacios",
        "Palomino",
        "Palomo",
        "Pantoja",
        "Paredes",
        "Parra",
        "Partida",
        "Patiño",
        "Paz",
        "Pedraza",
        "Pedroza",
        "Pelayo",
        "Peña",
        "Perales",
        "Peralta",
        "Perea",
        "Peres",
        "Pérez",
        "Pichardo",
        "Piña",
        "Pineda",
        "Pizarro",
        "Polanco",
        "Ponce",
        "Porras",
        "Portillo",
        "Posada",
        "Prado",
        "Preciado",
        "Prieto",
        "Puente",
        "Puga",
        "Pulido",
        "Quesada",
        "Quezada",
        "Quiñones",
        "Quiñónez",
        "Quintana",
        "Quintanilla",
        "Quintero",
        "Quiroz",
        "Rael",
        "Ramírez",
        "Ramón",
        "Ramos",
        "Rangel",
        "Rascón",
        "Raya",
        "Razo",
        "Regalado",
        "Rendón",
        "Rentería",
        "Reséndez",
        "Reyes",
        "Reyna",
        "Reynoso",
        "Rico",
        "Rincón",
        "Riojas",
        "Ríos",
        "Rivas",
        "Rivera",
        "Rivero",
        "Robledo",
        "Robles",
        "Rocha",
        "Rodarte",
        "Rodrígez",
        "Rodríguez",
        "Rodríquez",
        "Rojas",
        "Rojo",
        "Roldán",
        "Rolón",
        "Romero",
        "Romo",
        "Roque",
        "Rosado",
        "Rosales",
        "Rosario",
        "Rosas",
        "Roybal",
        "Rubio",
        "Ruelas",
        "Ruiz",
        "Saavedra",
        "Sáenz",
        "Saiz",
        "Salas",
        "Salazar",
        "Salcedo",
        "Salcido",
        "Saldaña",
        "Saldivar",
        "Salgado",
        "Salinas",
        "Samaniego",
        "Sanabria",
        "Sanches",
        "Sánchez",
        "Sandoval",
        "Santacruz",
        "Santana",
        "Santiago",
        "Santillán",
        "Sarabia",
        "Sauceda",
        "Saucedo",
        "Segovia",
        "Segura",
        "Sepúlveda",
        "Serna",
        "Serrano",
        "Serrato",
        "Sevilla",
        "Sierra",
        "Sisneros",
        "Solano",
        "Solís",
        "Soliz",
        "Solorio",
        "Solorzano",
        "Soria",
        "Sosa",
        "Sotelo",
        "Soto",
        "Suárez",
        "Tafoya",
        "Tamayo",
        "Tamez",
        "Tapia",
        "Tejada",
        "Tejeda",
        "Téllez",
        "Tello",
        "Terán",
        "Terrazas",
        "Tijerina",
        "Tirado",
        "Toledo",
        "Toro",
        "Torres",
        "Tórrez",
        "Tovar",
        "Trejo",
        "Treviño",
        "Trujillo",
        "Ulibarri",
        "Ulloa",
        "Urbina",
        "Ureña",
        "Urías",
        "Uribe",
        "Urrutia",
        "Vaca",
        "Valadez",
        "Valdés",
        "Valdez",
        "Valdivia",
        "Valencia",
        "Valentín",
        "Valenzuela",
        "Valladares",
        "Valle",
        "Vallejo",
        "Valles",
        "Valverde",
        "Vanegas",
        "Varela",
        "Vargas",
        "Vásquez",
        "Vázquez",
        "Vega",
        "Vela",
        "Velasco",
        "Velásquez",
        "Velázquez",
        "Vélez",
        "Véliz",
        "Venegas",
        "Vera",
        "Verdugo",
        "Verduzco",
        "Vergara",
        "Viera",
        "Vigil",
        "Villa",
        "Villagómez",
        "Villalobos",
        "Villalpando",
        "Villanueva",
        "Villareal",
        "Villarreal",
        "Villaseñor",
        "Villegas",
        "Yáñez",
        "Ybarra",
        "Zambrano",
        "Zamora",
        "Zamudio",
        "Zapata",
        "Zaragoza",
        "Zarate",
        "Zavala",
        "Zayas",
        "Zedillo",
        "Zelaya",
        "Zepeda",
        "Zúñiga",
    )

    prefixes = ("Sr(a).", "Dr.", "Mtro.", "Lic.", "Ing.")



from typing import List, Optional, Tuple

from .. import Provider as BankProvider


def get_clabe_control_digit(clabe: str) -> int:
    """Generate the checksum digit for a CLABE.

    :param clabe: CLABE.
    :return: The CLABE checksum digit.
    """
    factors = [3, 7, 1]
    products: List[int] = []

    for i, digit in enumerate(clabe[:17]):
        products.append((int(digit) * factors[i % 3]) % 10)

    return (10 - sum(products)) % 10


def is_valid_clabe(clabe: str) -> bool:
    """Check if a CLABE is valid using the checksum.

    :param clabe: CLABE.
    :return: True if the CLABE is valid, False otherwise.
    """
    if len(clabe) != 18 or not clabe.isdigit():
        return False

    return get_clabe_control_digit(clabe) == int(clabe[-1])


class Provider(BankProvider):
    """Bank provider for ``es_MX`` locale."""

    banks: Tuple[str, ...] = (
        "ABC Capital, S.A. I.B.M.",
        "Acciones y Valores Banamex, S.A. de C.V., Casa de Bolsa",
        "Actinver Casa de Bolsa, S.A. de C.V.",
        "Akala, S.A. de C.V., Sociedad Financiera Popular",
        "American Express Bank (México), S.A.",
        "AXA Seguros, S.A. De C.V.",
        "B y B Casa de Cambio, S.A. de C.V.",
        "Banca Afirme, S.A.",
        "Banca Mifel, S.A.",
        "Banco Actinver, S.A.",
        "Banco Ahorro Famsa, S.A.",
        "Banco Autofin México, S.A.",
        "Banco Azteca, S.A.",
        "Banco BASE, S.A. de I.B.M.",
        "Banco Compartamos, S.A.",
        "Banco Credit Suisse (México), S.A.",
        "Banco del Ahorro Nacional y Servicios Financieros, S.N.C.",
        "Banco del Bajío, S.A.",
        "Banco Inbursa, S.A.",
        "Banco Inmobiliario Mexicano, S.A., Institución de Banca Múltiple",
        "Banco Interacciones, S.A.",
        "Banco Invex, S.A.",
        "Banco J.P. Morgan, S.A.",
        "Banco Mercantil del Norte, S.A.",
        "Banco Monex, S.A.",
        "Banco Multiva, S.A.",
        "Banco Nacional de Comercio Exterior",
        "Banco Nacional de México, S.A.",
        "Banco Nacional de Obras y Servicios Públicos",
        "Banco Nacional del Ejército, Fuerza Aérea y Armada",
        "Banco PagaTodo S.A., Institución de Banca Múltiple",
        "Banco Regional de Monterrey, S.A.",
        "Banco Sabadell, S.A. I.B.M.",
        "Banco Santander, S.A.",
        "Banco Ve por Mas, S.A.",
        "Banco Wal Mart de México Adelante, S.A.",
        "BanCoppel, S.A.",
        "Bank of America México, S.A.",
        "Bank of Tokyo-Mitsubishi UFJ (México), S.A.",
        "Bankaool, S.A., Institución de Banca Múltiple",
        "Bansi, S.A.",
        "Barclays Bank México, S.A.",
        "BBVA Bancomer, S.A.",
        "Bulltick Casa de Bolsa, S.A. de C.V.",
        "Caja Popular Mexicana, S.C. de A.P. de R.L. De C.V.",
        "Casa de Bolsa Finamex, S.A. de C.V.",
        "Casa de Cambio Tíber, S.A. de C.V.",
        "CI Casa de Bolsa, S.A. de C.V.",
        "CLS Bank International",
        "Consubanco, S.A.",
        "Consultoría Internacional Banco, S.A.",
        "Consultoría Internacional Casa de Cambio, S.A. de C.V.",
        "Deutsche Bank México, S.A.",
        "Deutsche Securities, S.A. de C.V.",
        "Estructuradores del Mercado de Valores Casa de Bolsa, S.A. de C.V.",
        "Evercore Casa de Bolsa, S.A. de C.V.",
        "Financiera Nacional De Desarrollo Agropecuario, Rural, F y P.",
        "Fincomún, Servicios Financieros Comunitarios, S.A. de C.V.",
        "GBM Grupo Bursátil Mexicano, S.A. de C.V.",
        "GE Money Bank, S.A.",
        "HDI Seguros, S.A. de C.V.",
        "Hipotecaria su Casita, S.A. de C.V.",
        "HSBC México, S.A.",
        "Industrial and Commercial Bank of China, S.A., Institución de Banca Múltiple",
        "ING Bank (México), S.A.",
        "Inter Banco, S.A.",
        "Intercam Casa de Bolsa, S.A. de C.V.",
        "Intercam Casa de Cambio, S.A. de C.V.",
        "Inversora Bursátil, S.A. de C.V.",
        "IXE Banco, S.A.",
        "J.P. Morgan Casa de Bolsa, S.A. de C.V.",
        "J.P. SOFIEXPRESS, S.A. de C.V., S.F.P.",
        "Kuspit Casa de Bolsa, S.A. de C.V.",
        "Libertad Servicios Financieros, S.A. De C.V.",
        "MAPFRE Tepeyac S.A.",
        "Masari Casa de Bolsa, S.A.",
        "Merrill Lynch México, S.A. de C.V., Casa de Bolsa",
        "Monex Casa de Bolsa, S.A. de C.V.",
        "Multivalores Casa de Bolsa, S.A. de C.V. Multiva Gpo. Fin.",
        "Nacional Financiera, S.N.C.",
        "Opciones Empresariales Del Noreste, S.A. DE C.V.",
        "OPERADORA ACTINVER, S.A. DE C.V.",
        "Operadora De Pagos Móviles De México, S.A. De C.V.",
        "Operadora de Recursos Reforma, S.A. de C.V.",
        "OrderExpress Casa de Cambio , S.A. de C.V. AAC",
        "Profuturo G.N.P., S.A. de C.V.",
        "Scotiabank Inverlat, S.A.",
        "SD. INDEVAL, S.A. de C.V.",
        "Seguros Monterrey New York Life, S.A de C.V.",
        "Sistema de Transferencias y Pagos STP, S.A. de C.V., SOFOM E.N.R.",
        "Skandia Operadora S.A. de C.V.",
        "Skandia Vida S.A. de C.V.",
        "Sociedad Hipotecaria Federal, S.N.C.",
        "Solución Asea, S.A. de C.V., Sociedad Financiera Popular",
        "Sterling Casa de Cambio, S.A. de C.V.",
        "Telecomunicaciones de México",
        "The Royal Bank of Scotland México, S.A.",
        "UBS Banco, S.A.",
        "UNAGRA, S.A. de C.V., S.F.P.",
        "Única Casa de Cambio, S.A. de C.V.",
        "Valores Mexicanos Casa de Bolsa, S.A. de C.V.",
        "Valué, S.A. de C.V., Casa de Bolsa",
        "Vector Casa de Bolsa, S.A. de C.V.",
        "Volkswagen Bank S.A. Institución de Banca Múltiple",
        "Zúrich Compañía de Seguros, S.A.",
        "Zúrich Vida, Compañía de Seguros, S.A.",
    )

    bank_codes: Tuple[int, ...] = (
        2,
        6,
        9,
        12,
        14,
        19,
        21,
        22,
        30,
        32,
        36,
        37,
        42,
        44,
        58,
        59,
        60,
        62,
        72,
        102,
        103,
        106,
        108,
        110,
        112,
        113,
        116,
        124,
        126,
        127,
        128,
        129,
        130,
        131,
        132,
        133,
        134,
        135,
        136,
        137,
        138,
        139,
        140,
        141,
        143,
        145,
        147,
        148,
        150,
        155,
        156,
        166,
        168,
        600,
        601,
        602,
        604,
        605,
        606,
        607,
        608,
        610,
        611,
        613,
        614,
        615,
        616,
        617,
        618,
        619,
        620,
        621,
        622,
        623,
        624,
        626,
        627,
        628,
        629,
        630,
        631,
        632,
        633,
        634,
        636,
        637,
        638,
        640,
        642,
        646,
        647,
        648,
        649,
        651,
        652,
        653,
        655,
        656,
        659,
        670,
        674,
        677,
        679,
        684,
        901,
        902,
    )

    def clabe(self, bank_code: Optional[int] = None) -> str:
        """Generate a mexican bank account CLABE.

        Sources:

        - https://en.wikipedia.org/wiki/CLABE

        :return: A fake CLABE number.

        :sample:
        :sample: bank_code=2
        """
        bank = bank_code or self.random_element(self.bank_codes)
        city = self.random_int(0, 999)
        branch = self.random_int(0, 9999)
        account = self.random_int(0, 9999999)

        result = f"{bank:03d}{city:03d}{branch:04d}{account:07d}"
        control_digit = get_clabe_control_digit(result)

        return result + str(control_digit)



from typing import Tuple

from .. import Provider as AddressProvider


class Provider(AddressProvider):
    """Address provider for ar_DZ locale."""

    # Source: https://ar.wikipedia.org/wiki/ولايات_الجزائر
    wilayas: Tuple[str, ...] = (
        "أدرار",
        "الشلف",
        "الأغواط",
        "أم البواقي",
        "باتنة",
        "بجاية",
        "بسكرة",
        "بشار",
        "البليدة",
        "البويرة",
        "تمنراست",
        "تبسة",
        "تلمسان",
        "تيارت",
        "تيزي وزو",
        "الجزائر",
        "الجلفة",
        "جيجل",
        "سطيف",
        "سعيدة",
        "سكيكدة",
        "سيدي بلعباس",
        "عنابة",
        "قالمة",
        "قسنطينة",
        "المدية",
        "مستغانم",
        "المسيلة",
        "معسكر",
        "ورقلة",
        "وهران",
        "البيض",
        "إليزي",
        "برج بوعريريج",
        "بومرداس",
        "الطارف",
        "تندوف",
        "تيسمسيلت",
        "الوادي",
        "خنشلة",
        "سوق أهراس",
        "تيبازة",
        "ميلة",
        "عين الدفلى",
        "النعامة",
        "عين تموشنت",
        "غرداية",
        "غليزان",
        "تيميمون",
        "برج باجي مختار",
        "أولاد جلال",
        "بني عباس",
        "عين صالح",
        "عين قزام",
        "تقرت",
        "جانت",
        "المغير",
        "المنيعة",
    )

    # Source: https://github.com/othmanus/algeria-cities
    cities: Tuple[str, ...] = (
        # Wilaya 01 - أدرار
        "أدرار",
        "رقان",
        "أولف",
        "تيميمون",
        # Wilaya 02 - الشلف
        "الشلف",
        "تنس",
        "بوقادير",
        # Wilaya 03 - الأغواط
        "الأغواط",
        "أفلو",
        "عين ماضي",
        # Wilaya 04 - أم البواقي
        "أم البواقي",
        "عين مليلة",
        "عين البيضاء",
        # Wilaya 05 - باتنة
        "باتنة",
        "بريكة",
        "عين التوتة",
        "أريس",
        "نقاوس",
        # Wilaya 06 - بجاية
        "بجاية",
        "أقبو",
        "خراطة",
        "تاكسلانت",
        "أميزور",
        # Wilaya 07 - بسكرة
        "بسكرة",
        "طولقة",
        "سيدي عقبة",
        "زريبة الوادي",
        # Wilaya 08 - بشار
        "بشار",
        "تاغيت",
        "القنادسة",
        # Wilaya 09 - البليدة
        "البليدة",
        "بوفاريك",
        "الأربعاء",
        "بوعينان",
        # Wilaya 10 - البويرة
        "البويرة",
        "سور الغزلان",
        "عين بسام",
        # Wilaya 11 - تمنراست
        "تمنراست",
        "أبلسة",
        # Wilaya 12 - تبسة
        "تبسة",
        "بئر العاتر",
        "الونزة",
        # Wilaya 13 - تلمسان
        "تلمسان",
        "مغنية",
        "الغزوات",
        "ندرومة",
        "مرسى بن مهيدي",
        # Wilaya 14 - تيارت
        "تيارت",
        "السوقر",
        "فرندة",
        "تاقدمت",
        # Wilaya 15 - تيزي وزو
        "تيزي وزو",
        "ذراع بن خدة",
        "أزفون",
        "بوغني",
        "تيقزيرت",
        # Wilaya 16 - الجزائر
        "الجزائر الوسطى",
        "باب الوادي",
        "القصبة",
        "سيدي امحمد",
        "حسين داي",
        "القبة",
        "الحراش",
        "الرويبة",
        "زرالدة",
        "بئر مراد رايس",
        "بوزريعة",
        "الشراقة",
        "الدرارية",
        "باب الزوار",
        "برج الكيفان",
        "الدار البيضاء",
        "عين طاية",
        # Wilaya 17 - الجلفة
        "الجلفة",
        "مسعد",
        "عين وسارة",
        "حاسي بحبح",
        # Wilaya 18 - جيجل
        "جيجل",
        "الطاهير",
        "الميلية",
        "زيامة منصورية",
        # Wilaya 19 - سطيف
        "سطيف",
        "العلمة",
        "جميلة",
        "عين أرنات",
        "قجال",
        # Wilaya 20 - سعيدة
        "سعيدة",
        "عين الحجر",
        # Wilaya 21 - سكيكدة
        "سكيكدة",
        "عزابة",
        "الحروش",
        "القل",
        # Wilaya 22 - سيدي بلعباس
        "سيدي بلعباس",
        "عين البرد",
        "مرحوم",
        "تلموني",
        # Wilaya 23 - عنابة
        "عنابة",
        "البوني",
        "عين الباردة",
        "سيدي عمار",
        # Wilaya 24 - قالمة
        "قالمة",
        "بومهرة أحمد",
        "حمام النبايل",
        # Wilaya 25 - قسنطينة
        "قسنطينة",
        "الخروب",
        "ديدوش مراد",
        "زيغود يوسف",
        "عين عبيد",
        # Wilaya 26 - المدية
        "المدية",
        "قصر البخاري",
        "البرواقية",
        "بوغار",
        # Wilaya 27 - مستغانم
        "مستغانم",
        "سيدي علي",
        "عين تادلس",
        "مزغران",
        # Wilaya 28 - المسيلة
        "المسيلة",
        "بوسعادة",
        "سيدي عيسى",
        "عين الملح",
        # Wilaya 29 - معسكر
        "معسكر",
        "تيغنيف",
        "بوحنيفية",
        "سيق",
        # Wilaya 30 - ورقلة
        "ورقلة",
        "حاسي مسعود",
        "عين البيضاء",
        # Wilaya 31 - وهران
        "وهران",
        "أرزيو",
        "عين الترك",
        "بئر الجير",
        "مرسى الحجاج",
        "السانية",
        # Wilaya 32 - البيض
        "البيض",
        "بريزينة",
        # Wilaya 33 - إليزي
        "إيليزي",
        "إن أمناس",
        # Wilaya 34 - برج بوعريريج
        "برج بوعريرج",
        "رأس الوادي",
        "ثنية النصر",
        # Wilaya 35 - بومرداس
        "بومرداس",
        "خميس الخشنة",
        "برج منايل",
        "دلس",
        # Wilaya 36 - الطارف
        "الطارف",
        "القالة",
        "بن مهيدي",
        # Wilaya 37 - تندوف
        "تندوف",
        # Wilaya 38 - تيسمسيلت
        "تيسمسيلت",
        "خميستي",
        # Wilaya 39 - الوادي
        "الوادي",
        "قمار",
        "حاسي خليفة",
        "الرقيبة",
        # Wilaya 40 - خنشلة
        "خنشلة",
        "بابار",
        # Wilaya 41 - سوق أهراس
        "سوق أهراس",
        "مداوروش",
        "سدراتة",
        # Wilaya 42 - تيبازة
        "تيبازة",
        "شرشال",
        "بواسماعيل",
        "القليعة",
        # Wilaya 43 - ميلة
        "ميلة",
        "فرجيوة",
        "شلغوم العيد",
        # Wilaya 44 - عين الدفلى
        "عين الدفلى",
        "خميس مليانة",
        "العطاف",
        "مليانة",
        # Wilaya 45 - النعامة
        "النعامة",
        "المشرية",
        "عين الصفراء",
        # Wilaya 46 - عين تموشنت
        "عين تموشنت",
        "بني صاف",
        "حمام بوحجر",
        # Wilaya 47 - غرداية
        "غرداية",
        "القرارة",
        "متليلي",
        "العطف",
        # Wilaya 48 - غليزان
        "غليزان",
        "مازونة",
        "عمي موسى",
        # New wilayas (49–58)
        "تيميمون",
        "برج باجي مختار",
        "أولاد جلال",
        "بني عباس",
        "عين صالح",
        "عين قزام",
        "تقرت",
        "جانت",
        "المغير",
        "المنيعة",
    )

    street_prefixes: Tuple[str, ...] = ("شارع", "نهج", "طريق", "حي", "زنقة", "دوار", "بلدية")

    building_number_formats: Tuple[str, ...] = ("#", "##", "###")

    # Source: https://github.com/othmanus/algeria-cities
    postcode_formats: Tuple[str, ...] = tuple(f"{i:02d}###" for i in range(1, 49))

    street_name_formats: Tuple[str, ...] = (
        "{{street_prefix}} {{last_name}}",
        "{{street_prefix}} {{first_name}} {{last_name}}",
    )

    street_address_formats: Tuple[str, ...] = ("{{building_number}} {{street_name}}",)

    address_formats: Tuple[str, ...] = (
        "{{street_address}} {{city}}",
        "{{street_address}} - {{city}}",
        "{{street_address}} {{city}} {{wilaya}}",
        "{{street_address}} {{city}} - {{wilaya}}",
    )

    def street_prefix(self) -> str:
        return self.random_element(self.street_prefixes)

    def city(self) -> str:
        return self.random_element(self.cities)

    def administrative_unit(self) -> str:
        return self.random_element(self.wilayas)

    wilaya = administrative_unit



from .. import Provider as PersonProvider


class Provider(PersonProvider):

    formats_male = (
        "{{first_name_male}} {{last_name}}",
        "{{prefix_male}} {{first_name_male}} {{last_name}}",
    )

    formats_female = (
        "{{first_name_female}} {{last_name}}",
        "{{prefix_female}} {{first_name_female}} {{last_name}}",
    )

    formats = (
        "{{first_name}} {{last_name}}",
        "{{prefix}} {{first_name}} {{last_name}}",
    )

    # names taken from https://www.behindthename.com/names/gender/feminine/usage/gujarati
    first_names_female = (
        "અંકિતા",
        "અવની",
        "હીરા",
        "કાજલ",
        "કિરણ",
        "નેહા",
        "નિશા",
        "પૂજા",
        "પ્રાચી",
        "પ્રીતિ",
        "પ્રીતિ",
        "પૂજા",
        "રચના",
        "રાધીકા",
        "શ્રેયા",
        "શ્વેતા",
        "સોનલ",
        "તન્વી",
        "તેજલ",
        "ઉર્વી",
        "વર્ષા",
    )

    # names taken from https://www.behindthename.com/names/gender/masculine/usage/gujarati
    first_names_male = (
        "અભિષેક",
        "અજય",
        "અક્ષય",
        "આનંદ",
        "અનિલ",
        "અંકિત",
        "અર્જુન",
        "અરુણ",
        "આશિષ",
        "અશોક",
        "ભારત",
        "બ્રિજેશ",
        "ચેતન",
        "ચિરાગ",
        "દર્શન",
        "દીપા",
        "દીપક",
        "ધવલ",
        "દિલીપ",
        "દિનેશ",
        "દીપા",
        "દીપક",
        "હરીશ",
        "હર્ષ",
        "હર્ષલ",
        "હીરા",
        "જગદીશ",
        "જય",
        "જયેશ",
        "જિતેન્દ્ર",
        "કાજલ",
        "કમલ",
        "કરણ",
        "કિરણ",
        "કિશન",
        "કૃષ્ણ",
        "કુમાર",
        "કુનાલ",
        "મહેન્દ્ર",
        "મહેશ",
        "મનોજ",
        "મયૂર",
        "મિતુલ",
        "મુકેશ",
        "નરેન્દ્ર",
        "નીરજ",
        "નિખિલ",
        "નીરજ",
        "નીરવ",
        "નિશાંત",
        "નિતિન",
        "પંકજ",
        "પાર્થ",
        "પ્રકાશ",
        "પ્રણવ",
        "પ્રતિક",
        "પ્રતિક",
        "પ્રવીણ",
        "પ્રવીણ",
        "રાહુલ",
        "રાજ",
        "રાજેન્દ્ર",
        "રાજેશ",
        "રાકેશ",
        "રમેશ",
        "રવિ",
        "રોહિત",
        "સચિન",
        "સમીર",
        "સમીર",
        "સંદિપ",
        "સંદિપ",
        "સંજય",
        "સંજીવ",
        "સંજીવ",
        "શેખર",
        "સિદ્ધાર્થ",
        "સુભાષ",
        "સુનીલ",
        "સૂરજ",
        "તુષાર",
        "વસંત",
        "વિક્રમ",
        "વિપુલ",
        "વિરાજ",
        "વિશાલ",
        "વિવેક",
        "યશ",
    )

    first_names = first_names_female + first_names_male

    # last names taken from https://surnames.behindthename.com/names/usage/gujarati
    last_names = (
        "ચૌધરી",
        "ચૌધરી",
        "ગઢવી",
        "ગુપ્તા",
        "જૈન",
        "જોષી",
        "કુમાર",
        "પટેલ",
        "શર્મા",
    )

    prefixes_female = ("શ્રીમતી", "કુમારી")

    prefixes_male = ("શ્રી", "શ્રી માન")

    prefixes = prefixes_female + prefixes_male



from .. import Provider as BankProvider


class Provider(BankProvider):
    """Implement bank provider for ``de_CH`` locale."""

    bban_format = "#################"
    country_code = "CH"

    # Major Swiss banks - Source: https://de.wikipedia.org/wiki/Schweizer_Bankwesen
    banks = (
        "UBS",
        "Credit Suisse",
        "Raiffeisen Schweiz",
        "Zürcher Kantonalbank",
        "PostFinance",
        "Julius Bär",
        "Banque Cantonale Vaudoise",
        "Migros Bank",
        "Basler Kantonalbank",
        "Luzerner Kantonalbank",
        "Union Bancaire Privée",
        "Vontobel",
    )



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}}-{{last_name}} {{company_suffix}}",
        "{{last_name}}, {{last_name}} en {{last_name}} {{company_suffix}}",
    )
    company_suffixes = ("NV", "BV", "CV", "VOF", "CommV")



from ..es import Provider as CurrencyProvider


class Provider(CurrencyProvider):
    price_formats = ["%##", "%.###", "%#.##0", "%##.##0", "%##.##0", "%.###.##0"]

    def pricetag(self) -> str:
        return "\N{DOLLAR SIGN}\N{NO-BREAK SPACE}" + self.numerify(self.random_element(self.price_formats))



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    DAY_NAMES = {
        "0": "Dushanba",
        "1": "Seshanba",
        "2": "Chorshanba",
        "3": "Payshanba",
        "4": "Juma",
        "5": "Shanba",
        "6": "Yakshanba",
    }

    MONTH_NAMES = {
        "01": "Yanvar",
        "02": "Fevral",
        "03": "Mart",
        "04": "Aprel",
        "05": "May",
        "06": "Iyun",
        "07": "Iyul",
        "08": "Avgust",
        "09": "Sentabr",
        "10": "Oktabr",
        "11": "Noyabr",
        "12": "Dekabr",
    }

    def day_of_week(self) -> str:
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self) -> str:
        month = self.month()
        return self.MONTH_NAMES[month]



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):
    price_formats = ["#,##", "%#,##", "%##,##", "%.###,##", "%#.###,##"]

    def pricetag(self):
        return self.numerify(self.random_element(self.price_formats)) + "\N{NO-BREAK SPACE}\N{EURO SIGN}"



from faker.utils.checksums import calculate_luhn

from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}}-{{last_name}} {{company_suffix}}",
        "{{last_name}}, {{last_name}} e {{last_name}} {{company_suffix}}",
    )

    catch_phrase_words = (
        (
            "Abilità",
            "Access",
            "Adattatore",
            "Algoritmo",
            "Alleanza",
            "Analizzatore",
            "Applicazione",
            "Approccio",
            "Architettura",
            "Archivio",
            "Intelligenza artificiale",
            "Array",
            "Attitudine",
            "Benchmark",
            "Capacità",
            "Sfida",
            "Circuito",
            "Collaborazione",
            "Complessità",
            "Concetto",
            "Conglomerato",
            "Contingenza",
            "Core",
            "Database",
            "Data-warehouse",
            "Definizione",
            "Emulazione",
            "Codifica",
            "Criptazione",
            "Firmware",
            "Flessibilità",
            "Previsione",
            "Frame",
            "framework",
            "Funzione",
            "Funzionalità",
            "Interfaccia grafica",
            "Hardware",
            "Help-desk",
            "Gerarchia",
            "Hub",
            "Implementazione",
            "Infrastruttura",
            "Iniziativa",
            "Installazione",
            "Set di istruzioni",
            "Interfaccia",
            "Soluzione internet",
            "Intranet",
            "Conoscenza base",
            "Matrici",
            "Matrice",
            "Metodologia",
            "Middleware",
            "Migrazione",
            "Modello",
            "Moderazione",
            "Monitoraggio",
            "Moratoria",
            "Rete",
            "Architettura aperta",
            "Sistema aperto",
            "Orchestrazione",
            "Paradigma",
            "Parallelismo",
            "Policy",
            "Portale",
            "Struttura di prezzo",
            "Prodotto",
            "Produttività",
            "Progetto",
            "Proiezione",
            "Protocollo",
            "Servizio clienti",
            "Software",
            "Soluzione",
            "Standardizzazione",
            "Strategia",
            "Struttura",
            "Successo",
            "Sovrastruttura",
            "Supporto",
            "Sinergia",
            "Task-force",
            "Finestra temporale",
            "Strumenti",
            "Utilizzazione",
            "Sito web",
            "Forza lavoro",
        ),
        (
            "adattiva",
            "avanzata",
            "migliorata",
            "assimilata",
            "automatizzata",
            "bilanciata",
            "centralizzata",
            "compatibile",
            "configurabile",
            "cross-platform",
            "decentralizzata",
            "digitalizzata",
            "distribuita",
            "piccola",
            "ergonomica",
            "esclusiva",
            "espansa",
            "estesa",
            "configurabile",
            "fondamentale",
            "orizzontale",
            "implementata",
            "innovativa",
            "integrata",
            "intuitiva",
            "inversa",
            "gestita",
            "obbligatoria",
            "monitorata",
            "multi-canale",
            "multi-laterale",
            "open-source",
            "operativa",
            "ottimizzata",
            "organica",
            "persistente",
            "polarizzata",
            "proattiva",
            "programmabile",
            "progressiva",
            "reattiva",
            "riallineata",
            "ricontestualizzata",
            "ridotta",
            "robusta",
            "sicura",
            "condivisibile",
            "stand-alone",
            "switchabile",
            "sincronizzata",
            "sinergica",
            "totale",
            "universale",
            "user-friendly",
            "versatile",
            "virtuale",
            "visionaria",
        ),
        (
            "24 ore",
            "24/7",
            "terza generazione",
            "quarta generazione",
            "quinta generazione",
            "sesta generazione",
            "asimmetrica",
            "asincrona",
            "background",
            "bi-direzionale",
            "biforcata",
            "bottom-line",
            "coerente",
            "coesiva",
            "composita",
            "sensibile al contesto",
            "basta sul contesto",
            "basata sul contenuto",
            "dedicata",
            "didattica",
            "direzionale",
            "discreta",
            "dinamica",
            "eco-centrica",
            "esecutiva",
            "esplicita",
            "full-range",
            "globale",
            "euristica",
            "alto livello",
            "olistica",
            "omogenea",
            "ibrida",
            "impattante",
            "incrementale",
            "intangibile",
            "interattiva",
            "intermediaria",
            "locale",
            "logistica",
            "massimizzata",
            "metodica",
            "mission-critical",
            "mobile",
            "modulare",
            "motivazionale",
            "multimedia",
            "multi-tasking",
            "nazionale",
            "neutrale",
            "nextgeneration",
            "non-volatile",
            "object-oriented",
            "ottima",
            "ottimizzante",
            "radicale",
            "real-time",
            "reciproca",
            "regionale",
            "responsiva",
            "scalabile",
            "secondaria",
            "stabile",
            "statica",
            "sistematica",
            "sistemica",
            "tangibile",
            "terziaria",
            "uniforme",
            "valore aggiunto",
        ),
    )

    bsWords = (
        (
            "partnerships",
            "comunità",
            "ROI",
            "soluzioni",
            "e-services",
            "nicchie",
            "tecnologie",
            "contenuti",
            "supply-chains",
            "convergenze",
            "relazioni",
            "architetture",
            "interfacce",
            "mercati",
            "e-commerce",
            "sistemi",
            "modelli",
            "schemi",
            "reti",
            "applicazioni",
            "metriche",
            "e-business",
            "funzionalità",
            "esperienze",
            "webservices",
            "metodologie",
        ),
        (
            "implementate",
            "utilizzo",
            "integrate",
            "ottimali",
            "evolutive",
            "abilitate",
            "reinventate",
            "aggregate",
            "migliorate",
            "incentivate",
            "monetizzate",
            "sinergizzate",
            "strategiche",
            "deploy",
            "marchi",
            "accrescitive",
            "target",
            "sintetizzate",
            "spedizioni",
            "massimizzate",
            "innovazione",
            "guida",
            "estensioni",
            "generate",
            "exploit",
            "transizionali",
            "matrici",
            "ricontestualizzate",
        ),
        (
            "valore aggiunto",
            "verticalizzate",
            "proattive",
            "forti",
            "rivoluzionari",
            "scalabili",
            "innovativi",
            "intuitivi",
            "strategici",
            "e-business",
            "mission-critical",
            "24/7",
            "globali",
            "B2B",
            "B2C",
            "granulari",
            "virtuali",
            "virali",
            "dinamiche",
            "magnetiche",
            "web",
            "interattive",
            "sexy",
            "back-end",
            "real-time",
            "efficienti",
            "front-end",
            "distributivi",
            "estensibili",
            "mondiali",
            "open-source",
            "cross-platform",
            "sinergiche",
            "out-of-the-box",
            "enterprise",
            "integrate",
            "di impatto",
            "wireless",
            "trasparenti",
            "next-generation",
            "cutting-edge",
            "visionari",
            "plug-and-play",
            "collaborative",
            "olistiche",
            "ricche",
        ),
    )

    company_suffixes = ("SPA", "e figli", "Group", "s.r.l.")

    def _random_vat_office(self) -> int:
        """
        Returns a random code identifying the VAT office needed to build a valid VAT with company_vat.

        See https://it.wikipedia.org/wiki/Partita_IVA#Tabella_degli_Uffici_IVA
        """
        val = self.random_int(1, 104)

        # handle special cases
        if val == 101:
            return 120
        elif val == 102:
            return 121
        elif val == 103:
            return 888
        elif val == 104:
            return 999
        # else: between 1 and 100 are all valid
        return val

    def company_vat(self) -> str:
        """
        Returns Italian VAT identification number (Partita IVA).
        """
        code = self.bothify("#######") + str(self._random_vat_office()).zfill(3)
        luhn_checksum = str(calculate_luhn(int(code)))
        return f"IT{code}{luhn_checksum}"
